/* Extractions list */
.table-extractions thead th {
	text-align: left;
	vertical-align: middle;
}

.table-extractions tbody td {
	text-align: left;
	vertical-align: middle !important;
}

.table-extractions thead th:nth-child(5) {
	text-align: right;
}
.table-extractions thead th:nth-child(6) {
	text-align: right;
}
.table-extractions thead th:nth-child(7) {
	text-align: right;
}
.table-extractions tbody td:nth-child(5) {
	text-align: right;
}
.table-extractions tbody td:nth-child(6) {
	text-align: right;
}
.table-extractions tbody td:nth-child(7) {
	text-align: right;
}
.table-extractions tbody td:nth-child(8) {
	text-align: right;
	white-space: nowrap;
}
.table-extractions tfoot th:nth-child(5) {
	text-align: right;
}
.table-extractions tfoot th:nth-child(6) {
	text-align: right;
}
.table-extractions tfoot th:nth-child(7) {
	text-align: right;
}

/* Extractions ledger */
.table-extraction-ledger thead th:nth-child(5) {
	text-align: right;
}
.table-extraction-ledger thead th:nth-child(6) {
	text-align: right;
}
.table-extraction-ledger thead th:nth-child(7) {
	text-align: right;
}
#table-extraction-ledger tbody td:nth-child(5) {
	text-align: right;
}
#table-extraction-ledger tbody td:nth-child(6) {
	text-align: right;
}
#table-extraction-ledger tbody td:nth-child(7) {
	text-align: right;
}
